----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 15.11.2016 15:18:51
-- Design Name: 
-- Module Name: traffic_moore - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

use IEEE.STD_LOGIC_unsigned.all;   --- cnt '+'
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity traffic_moore_delayed is
generic (N: natural := 3);    
port ( clk : in  STD_LOGIC;
       reset : in  STD_LOGIC;
       led : out  STD_LOGIC_VECTOR (N-1 downto 0));
end traffic_moore_delayed;

architecture Behavioral of traffic_moore_delayed is
	type traff_state_type is (s0, s1, s2, s3); -- P, PS, Z ,S -> P
	signal state_reg, state_next: traff_state_type;
	--constants for delay
    constant C_SEC6: natural := 6;  
    constant C_SEC3: natural := 3;
        
    
    signal cnt: STD_LOGIC_VECTOR(2 downto 0):= (others => '0'); 

begin
	-- state register
   process(clk, reset)
   begin
      if (reset='1') then
         state_reg <= s0;
	  elsif rising_edge(clk) then
         case state_reg is
            when s0 =>
                if cnt < C_SEC6-1 then
                    state_reg <= s0;
                    cnt <= cnt + 1;
                else
                    state_reg <= s1;
                    cnt <= (others => '0');     
                end if;
            when s1 =>
                if cnt < C_SEC3-1 then
                    state_reg <= s1;
                    cnt <= cnt + 1;
                else 
                    state_reg <= s2;
                    cnt <= (others => '0');     
                end if;
            when s2 =>
                if cnt < C_SEC6-1 then
                    state_reg <= s2;
                    cnt <= cnt + 1;
                else 
                    state_reg <= s3;
                    cnt <= (others => '0');     
                end if;
            when s3 =>
                if cnt < C_SEC3-1 then
                    state_reg <= s3;
                    cnt <= cnt + 1;
                else 
                    state_reg <= s0;
                    cnt <= (others => '0');     
                end if;
            end case;
        end if;
    end process;
	
-- Moore output logic (led2, led1, led0)
   process(state_reg)
   begin
      case state_reg is
         when s0 =>
            led(N-1 downto 0) <= "100"; --P
         when s1 =>
            led(N-1 downto 0) <= "110"; --PS
	     when s2  =>
	        led(N-1 downto 0) <= "001"; --Z
	     when s3 =>
	        led(N-1 downto 0) <= "010"; --S
         end case;
   end process;
end Behavioral;
